<?php

namespace BotKit\Common;

// Команды бота

use BotKit\Common\Message;
use BotKit\Enums\Platform;
use BotKit\Enums\State;
use BotKit\Attachments\ImageAttachment;

use BotKit\Enums\GroupSelect;

use BotKit\Keyboards\TermsOfServiceKeyboard;
use BotKit\Keyboards\SelectUserTypeKeyboard;
use BotKit\Keyboards\HubKeyboard;
use BotKit\Keyboards\SelectGroupCourseKeyboard;
use BotKit\Keyboards\AskEnterCredentialsKeyboard;

use BotKit\Models;

class Commands {

    #region Callback

    // Отправка условий использования
    public static function showTermsOfService($e, $u, $driver) {
        $driver->editMessage(
            $e->getMessageID(),
            Message::create("[Условия использования]")
        );
    }

    // Выбор группы
    public static function handleGroupSelection($e, $u, $driver) {
        $params = $e->getParams();
        $intent = GroupSelect::from($params['intent']);
        switch ($intent) {
            case GroupSelect::Register:
                // Группа выбрана, зарегистрирован студент
                Models\StudentModel::create($u['id'], $params['group_id'], 0);
                
                $driver->editMessage(
                    $e->getMessageID(),
                    Message::create("Хочешь ввести логин и пароль для просмотра оценок?")
                        ->withKeyboard(new AskEnterCredentialsKeyboard())
                );
                break;
            default:
                $driver->sendMessage($u, Message::create("[Unknown group select value]"));
                break;
        }
    }

    // Выбор хочу/не хочу
    public static function enterCredentials($e, $u, $driver) {
        $params = $e->getParams();
        if ($params['enters'] == 0) {
            // Пользователь не захотел вводить пароль и логин
            $driver->editMessage(
                $e->getMessageID(),
                Message::create("Регистрация завершена!")
                    ->withKeyboard(new HubKeyboard())
            );
            $u->setState(State::Hub);
        } else {
            // Пользователь захотел вводить пароль и логин
            $driver->sendMessage($u, Message::create("Введи логин"));
            $u->setState(State::RegLogin);
        }
    }

    #endregion

    public static function helloWorld($e, $u, $driver) {
        //~ if ($u->getState() != State::HelloWorld) {
            //~ $driver->reply(
                //~ $e,
                //~ Message::create("Ты уже зарегистрирован")
                //~ ->withKeyboard(new HubKeyboard())
            //~ );
            //~ return;
        //~ }

        $driver->reply(
            $e,
            Message::create("Привет, я - Техбот. Моя задача - облегчить твою жизнь, но, для начала, мне нужно задать несколько вопросов"),
            false
        );
        $driver->reply(
            $e,
            Message
                ::create("Ознакомься с условиями использования прежде чем использовать мои функции")
                ->withKeyboard(new TermsOfServiceKeyboard()),
            false
        );
        $driver->reply(
            $e,
            Message
                ::create("Ты студент или преподаватель?")
                ->withKeyboard(new SelectUserTypeKeyboard()),
            false
        );
        $u->setState(State::RegSelectType);
    }

    // Обработка выбора типа аккаунта
    public static function handleSelectType($e, $u, $driver) {
        if ($e->getText() == "Я студент") {
            // Создать объект студента
            $driver->reply(
                $e,
                Message::create("Выбери курс, на котором сейчас учишься")
                    ->withKeyboard(new SelectGroupCourseKeyboard(GroupSelect::Register)),
                false
            );
            $u->setState(State::NoResponse);
        } else {
            // TODO
            $driver->reply($e, Message::create("Функционал ещё не реализован"));
        }
    }

    // Обработка ввода логина при регистрации
    /*
    public static function handleRegLogin($e, $u, $driver) {
        $student = Models\StudentModel::where([
            ['user_id', '=', $u['id']]
        ])[0];

        $student[
    }
    */

    public static function fallback($e, $u, $driver) {
        $driver->sendMessage($u, Message::create("[Fallback]: Ни один обработчик не сработал. Сообщите об этом сообщении администратору"));
    }

    public static function unknownEvent($e, $u, $driver) {
        $driver->sendMessage($u, Message::create("[Fallback]: Неизвестное событие. Сообщите об этом сообщении администратору"));
    }
}
