<?php
// Класс сообщения

namespace BotKit\Common;

use BotKit\Attachments\ImageAttachment;

class Message {

	// ID сообщения
	public $id;

	// Текст сообщения
	private string $text;

	// Клавиатура сообщения (если присутствует)
	private $keyboard;

	// Вложения сообщения
	private array $image_attachments = [];

	// Имеет ли сообщение изображения
	private bool $has_images = false;

	// Имеет ли сообщение клавиатуру
	private bool $has_keyboard = false;

	public function __construct($text) {
		$this->text = $text;
	}

	public static function create($text="") {
		return new self($text);
	}

	// Возвращает ID сообщения
	public function getID() {
		return $this->id;
	}

	// Возвращает текст сообщения
	public function getText() : string {
		return $this->text;
	}

	public function getKeyboard() {
		return $this->keyboard;
	}

	// Добавляет клавиатуру к сообщению
	public function withKeyboard($keyboard) {
		$this->has_keyboard = true;
		$this->keyboard = $keyboard;
		return $this;
	}

	// Добавляет вложение
	public function withImage(ImageAttachment $image) {
		$this->has_images = true;
		$this->image_attachments[] = $image;
		return $this;
	}

	// Возвращает массив всех сохранённых изображений
	public function getImages() {
		return $this->image_attachments;
	}

	// Возвращает true если вложения сообщения содержат изображения
	public function hasImages() : bool {
		return $this->has_images;
	}
	
	// Возвращает true если сообщение содержит клавиатуру
	public function hasKeyboard() : bool {
		return $this->has_keyboard;
	}
	
}