<?php
// Интерфейс для драйверов ботов

namespace BotKit\Drivers;

use BotKit\Common\Chat;
use BotKit\Common\Message;
use BotKit\Common\EventData;
use BotKit\Entities\User;
use BotKit\Events\Event;
use BotKit\Events\PlainMessageEvent;

interface Driver {

    // Возвращает true, если драйвер считает, что именно ему необходимо обработать этот запрос
    public function forThis() : bool;

    // Возвращает событие на основании данных запроса
    public function getEvent() : Event;

    // Отвечает на текстовое сообщение
    // Если empathise=true, сообщение должно быть явным ответом
    // $e - событие, на которое создаётся ответ
    // $msg - сообщение ответа
    public function reply(PlainMessageEvent $e, Message $msg, bool $empathise = true);

    // Отсылает сообщение пользователю
    public function sendMessage(User $u, Message $msg);

    // Изменяет сообщение
    // messageId - ID сообщения на платформе
    // msg - новые данные сообщения
    public function editMessage($messageId, Message $msg) : void;

    // Отправляет сообщение в чат
    public function sendToChat(Chat $chat, Message $msg);

    // Возвращает ник пользователя, например @aquadim
    public function getUserNick(User $u) : string;

    // Событие перед началом обработки запроса
    public function onProcessStart() : void;

    // Событие после ensureDriversLoaded
    public function onSelected() : void;

    // Событие завершения обработки
    public function onProcessEnd() : void;

    // Событие сохранения пользователя
    public function onUserSave(User $user) : void;

    // Вывод отладочного сообщения
    public function varDump(string $title, $variable) : void;
}