<?php

namespace BotKit\Entities;

use BotKit\Enums\State;
use Doctrine\ORM\Mapping as ORM;

#[ORM\Entity]
#[ORM\Table(name: 'user')]
class User {
    #[ORM\Id]
    #[ORM\Column(type: 'integer')]
    #[ORM\GeneratedValue]
    private int|null $id = null;

    // ID на платформе
    #[ORM\Column(type: 'integer')]
    private int $id_on_platform;

    // ID платформы
    #[ORM\Column(type: 'integer')]
    private int $platform_id;

    // Состояние пользователя
    #[ORM\Column(type: 'integer')]
    private int $state = State::HelloWorld->value;

    // Устанавливает ID на платформе
    public function setIdOnPlatform($id_on_platform) : void {
        $this->id_on_platform = $id_on_platform;
    }

    // Устанавливает ID платформы
    public function setPlatformId($platform_id) : void {
        $this->platform_id = $platform_id;
    }

    // Устанавливает состояние через перечисление
    public function setState(State $new_state) : void {
        $this->state = $new_state->value;
    }

    // Возвращает состояние
    public function getState() : State {
        return State::from($this->state);
    }

    // Устанавливает состояние через int
    public function setStateByInt(int $new_stateID) : void {
        $this->state = $new_stateID;
    }

    // Возвращает название состояния в котором находится пользователь
    public function getStateName() : string {
        return serialize(State::from($this->state));
    }

    // Возвращает true если состояние пользователя совпадает со $state
    public function inState(State $state) : bool {
        return $state->value == $this->state;
    }
}